# Turning on a light opens a serial port, which pulls the DTR line high.
# Turning off a light closes the serial port, pulling DTR low.

__all__ = ['SerialDtr']

class SerialDtr:
    def __init__(self, config):
        self.fn = config.get('serial_dtr', 'port')
        self.fd = None

    def tally_on(self):
        self.fd = open(self.fn)

    def tally_off(self):
        if self.fd:
            self.fd.close()
            self.fd = None

    def __del__(self):
        tally_off()



from .rpi_gpio import RpiGpio
from .serial_dtr import SerialDtr
from .stdout import Stdout
from .tomu import Tomu


PLUGINS = {
    'rpi_gpio': RpiGpio,
    'serial_dtr': SerialDtr,
    'stdout': Stdout,
    'tomu': Tomu,
}



# Makes a tomu.im USB Simple sample as a tally light.
# https://github.com/im-tomu/tomu-samples/tree/master/usb_simple

__all__ = ['TomuSimpleLed']

DO_USB = True
try:
    import usb.core
except ImportError:
    DO_USB = False


class Tomu:
    def __init__(self, config):
        self.on = int(config.get('tomu', 'on'))
        self.off = int(config.get('tomu', 'off'))
        if not DO_USB:
            raise ValueError('USB support not available. Install pyusb')

        self.device = usb.core.find(idVendor=0x1209, idProduct=0x70b1)
        if self.device is None:
            raise ValueError('Device not found, flash usb_simple on the tomu.')

        self.device.set_configuration()

    def tally_on(self):
        self.device.ctrl_transfer(0x40, 0, self.on, 0, '')

    def tally_off(self):
        self.device.ctrl_transfer(0x40, 0, self.off, 0, '')



#!/usr/bin/env python3
# Test driver for Voctolight plugins.
from argparse import ArgumentParser, FileType
from sys import exit

from lib.config import Config
from lib.plugins.all_plugins import PLUGINS

def main():
    parser = ArgumentParser(
        description='Test program for testing tallylight plugins')
    parser.add_argument(
        '-c', '--config', type=FileType('r'), help='Use a specific config file')
    args = parser.parse_args()
    config = Config(cmd_line_config=args.config)

    plugin_cls = PLUGINS.get(config.get('light', 'plugin'), None)
    if plugin_cls is None:
        print('No plugin selected, control will not work!')
        exit(1)

    plugin = plugin_cls(config)

    try:
        while True:
            print('Tally light on. Press ENTER to turn off, ^C to stop.')
            plugin.tally_on()
            input()
            print('Tally light off. Press ENTER to turn on, ^C to stop.')
            plugin.tally_off()
            input()
    except KeyboardInterrupt:
        pass

if __name__ in '__main__':
    main()



# Plugin to provide a tally light interface via stdout
__all__ = ['Stdout']

class Stdout:
    def __init__(self, config):
        pass

    def tally_on(self):
        print('Tally light on')

    def tally_off(self):
        print('Tally light off')



# Plugin to provide a tally light interface for a Raspberry Pi's GPIO
DO_GPIO = True
try:
    import RPi.GPIO as GPIO
    GPIO.setmode(GPIO.BOARD)
except ImportError:
    DO_GPIO = False

__all__ = ['RpiGpio']

class RpiGpio:
    def __init__(self, config):
        if not DO_GPIO:
            raise ValueError('RpiGpio will not work on this platform. Is RPi.GPIO installed?')

        all_gpios = [int(i) for i in config.getlist('rpi', 'gpios')]
        self.gpio_port = int(config.get('rpi', 'gpio_red'))

        GPIO.setup(all_gpios, GPIO.OUT)
        GPIO.output(all_gpios, GPIO.HIGH)

    def tally_on(self):
        GPIO.output(self.gpio_port, GPIO.LOW)

    def tally_off(self):
        GPIO.output(self.gpio_port, GPIO.HIGH)

    def __del__(self):
        GPIO.cleanup()



from configparser import ConfigParser
from os.path import join, dirname, realpath, expanduser

__all__ = ['Config']

class Config(ConfigParser):
    config_files = [
        join(dirname(realpath(__file__)),
                     'default-config.ini'),
        join(dirname(realpath(__file__)),
                     'config.ini'),
        '/etc/voctomix/voctolight.ini',
        '/etc/voctolight.ini',
        expanduser('~/.voctolight.ini'),
    ]

    def __init__(self, cmd_line_config=None):
        super().__init__()
        self.cmd_line_config = cmd_line_config
        self._read_config()

    def _read_config(self):
        self.read(self.config_files)
        if self.cmd_line_config:
            self.read_file(self.cmd_line_config)
            self.cmd_line_config.seek(0)

    def setup_with_server_config(self, server_config):
        self.clear()
        self._read_config()
        self.read_dict(server_config)

    def getlist(self, section, option):
        return [x.strip() for x in self.get(section, option).split(',')]
